@@ -4,11 +4,22 @@ from django.contrib.auth.models import User
 from django.db import models
 from django.utils.translation import ugettext as _
 
+from linaro_django_xmlrpc.models import AuthToken
 
 class JSONDataError(ValueError):
     """Error raised when JSON is syntactically valid but ill-formed."""
 
 
+class Tag(models.Model):
+
+    name = models.SlugField(unique=True)
+
+    description = models.TextField(null=True, blank=True)
+
+    def __unicode__(self):
+        return self.name
+
+
 class DeviceType(models.Model):
     """
     A class of device, for example a pandaboard or a snowball.
@@ -51,6 +62,8 @@ class Device(models.Model):
     current_job = models.ForeignKey(
         "TestJob", blank=True, unique=True, null=True)
 
+    tags = models.ManyToManyField(Tag, blank=True)
+
     status = models.IntegerField(
         choices = STATUS_CHOICES,
         default = IDLE,
@@ -123,6 +136,8 @@ class TestJob(models.Model):
         verbose_name = _(u"Submitter"),
     )
 
+    submit_token = models.ForeignKey(AuthToken, null=True)
+
     description = models.CharField(
         verbose_name = _(u"Description"),
         max_length = 200,
@@ -137,6 +152,8 @@ class TestJob(models.Model):
     requested_device_type = models.ForeignKey(
         DeviceType, null=True, default=None, related_name='+', blank=True)
 
+    tags = models.ManyToManyField(Tag, blank=True)
+
     # This is set once the job starts.
     actual_device = models.ForeignKey(
         Device, null=True, default=None, related_name='+', blank=True)
@@ -206,6 +223,8 @@ class TestJob(models.Model):
             definition=json_data, submitter=user, requested_device=target,
             requested_device_type=device_type, description=job_name)
         job.save()
+        for tag_name in job_data.get('device_tags', []):
+            job.tags.add(Tag.objects.get_or_create(name=tag_name)[0])
         return job
 
     def can_cancel(self, user):
