@@ -9,7 +9,9 @@ from django.test.client import Client
 
 from django_testscenarios.ubertest import TestCase
 
-from lava_scheduler_app.models import Device, DeviceType, TestJob
+from linaro_django_xmlrpc.models import AuthToken
+
+from lava_scheduler_app.models import Device, DeviceType, Tag, TestJob
 
 
 
@@ -26,6 +28,9 @@ class TestTransport(xmlrpclib.Transport):
         self._use_datetime = True
 
     def request(self, host, handler, request_body, verbose=0):
+        from django.conf import settings
+        # This is a total hack.  See bug 904054 for more.
+        settings.MOUNT_POINT = ''
         self.verbose = verbose
         response = self.client.post(
             handler, request_body, content_type="text/xml")
@@ -132,6 +137,41 @@ class TestTestJob(TestCaseWithFactory):
             json.dumps({'device_type':'panda'}), self.factory.make_user())
         self.assertEqual(job.status, TestJob.SUBMITTED)
 
+    def test_from_json_and_user_sets_no_tags_if_no_tags(self):
+        self.factory.ensure_device_type(name='panda')
+        job = TestJob.from_json_and_user(
+            json.dumps({'device_type':'panda', 'device_tags':[]}),
+            self.factory.make_user())
+        self.assertEqual(set(job.tags.all()), set([]))
+
+    def test_from_json_and_user_sets_tag_from_device_tags(self):
+        self.factory.ensure_device_type(name='panda')
+        job = TestJob.from_json_and_user(
+            json.dumps({'device_type':'panda', 'device_tags':['tag']}),
+            self.factory.make_user())
+        self.assertEqual(
+            set(tag.name for tag in job.tags.all()), set(['tag']))
+
+    def test_from_json_and_user_sets_multiple_tag_from_device_tags(self):
+        self.factory.ensure_device_type(name='panda')
+        job = TestJob.from_json_and_user(
+            json.dumps({'device_type':'panda', 'device_tags':['tag1', 'tag2']}),
+            self.factory.make_user())
+        self.assertEqual(
+            set(tag.name for tag in job.tags.all()), set(['tag1', 'tag2']))
+
+    def test_from_json_and_user_reuses_tag_objects(self):
+        self.factory.ensure_device_type(name='panda')
+        job1 = TestJob.from_json_and_user(
+            json.dumps({'device_type':'panda', 'device_tags':['tag']}),
+            self.factory.make_user())
+        job2 = TestJob.from_json_and_user(
+            json.dumps({'device_type':'panda', 'device_tags':['tag']}),
+            self.factory.make_user())
+        self.assertEqual(
+            set(tag.pk for tag in job1.tags.all()),
+            set(tag.pk for tag in job2.tags.all()))
+
 
 class TestSchedulerAPI(TestCaseWithFactory):
 
@@ -295,6 +335,53 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
             None,
             DatabaseJobSource().getJobForBoard_impl('panda02'))
 
+    def _makeBoardWithTags(self, tags):
+        board = self.factory.make_device()
+        for tag_name in tags:
+            board.tags.add(Tag.objects.get_or_create(name=tag_name)[0])
+        return board
+
+    def _makeJobWithTagsForBoard(self, tags, board):
+        job = self.factory.make_testjob(requested_device=board)
+        for tag_name in tags:
+            job.tags.add(Tag.objects.get_or_create(name=tag_name)[0])
+        return job
+
+    def assertBoardWithTagsGetsJobWithTags(self, board_tags, job_tags):
+        board = self._makeBoardWithTags(board_tags)
+        self._makeJobWithTagsForBoard(job_tags, board)
+        self.assertEqual(
+            board.hostname,
+            DatabaseJobSource().getJobForBoard_impl(board.hostname)['target'])
+
+    def assertBoardWithTagsDoesNotGetJobWithTags(self, board_tags, job_tags):
+        board = self._makeBoardWithTags(board_tags)
+        self._makeJobWithTagsForBoard(job_tags, board)
+        self.assertEqual(
+            None,
+            DatabaseJobSource().getJobForBoard_impl(board.hostname))
+
+    def test_getJobForBoard_does_not_return_job_if_board_lacks_tag(self):
+        self.assertBoardWithTagsDoesNotGetJobWithTags([], ['tag'])
+
+    def test_getJobForBoard_returns_job_if_board_has_tag(self):
+        self.assertBoardWithTagsGetsJobWithTags(['tag'], ['tag'])
+
+    def test_getJobForBoard_returns_job_if_board_has_both_tags(self):
+        self.assertBoardWithTagsGetsJobWithTags(['tag1', 'tag2'], ['tag1', 'tag2'])
+
+    def test_getJobForBoard_returns_job_if_board_has_extra_tags(self):
+        self.assertBoardWithTagsGetsJobWithTags(['tag1', 'tag2'], ['tag1'])
+
+    def test_getJobForBoard_does_not_return_job_if_board_has_only_one_tag(self):
+        self.assertBoardWithTagsDoesNotGetJobWithTags(['tag1'], ['tag1', 'tag2'])
+
+    def test_getJobForBoard_does_not_return_job_if_board_has_unrelated_tag(self):
+        self.assertBoardWithTagsDoesNotGetJobWithTags(['tag1'], ['tag2'])
+
+    def test_getJobForBoard_does_not_return_job_if_only_one_tag_matches(self):
+        self.assertBoardWithTagsDoesNotGetJobWithTags(['tag1', 'tag2'], ['tag1', 'tag3'])
+
     def test_getJobForBoard_sets_start_time(self):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(requested_device=device)
@@ -328,30 +415,114 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         device = Device.objects.get(pk=device.pk)
         self.assertEqual(job, device.current_job)
 
+    def test_getJobForBoard_creates_token(self):
+        device = self.factory.make_device(hostname='panda01')
+        job = self.factory.make_testjob(requested_device=device)
+        transaction.commit()
+        DatabaseJobSource().getJobForBoard_impl('panda01')
+        # reload from the database
+        job = TestJob.objects.get(pk=job.pk)
+        device = Device.objects.get(pk=device.pk)
+        self.assertIsNotNone(job.submit_token)
+        self.assertEqual(job.submitter, job.submit_token.user)
+
+    def test_getJobForBoard_inserts_target_into_json(self):
+        panda_type = self.factory.ensure_device_type(name='panda')
+        self.factory.make_device(hostname='panda01', device_type=panda_type)
+        definition = {'foo': 'bar'}
+        self.factory.make_testjob(
+            requested_device_type=panda_type,
+            definition=json.dumps(definition))
+        transaction.commit()
+        json_data = DatabaseJobSource().getJobForBoard_impl('panda01')
+        self.assertIn('target', json_data)
+        self.assertEqual('panda01', json_data['target'])
+
+    def test_getJobForBoard_inserts_submit_token_into_json(self):
+        panda_type = self.factory.ensure_device_type(name='panda')
+        self.factory.make_device(hostname='panda01', device_type=panda_type)
+        definition = {
+            'actions': [
+                {
+                    "command": "submit_results",
+                    "parameters":
+                    {
+                        "server": "http://test-server/RPC2/",
+                        "stream": "/private/personal/test/test/",
+                    }
+                }
+            ]
+        }
+        job = self.factory.make_testjob(
+            requested_device_type=panda_type,
+            definition=json.dumps(definition))
+        transaction.commit()
+        json_data = DatabaseJobSource().getJobForBoard_impl('panda01')
+        job = TestJob.objects.get(pk=job.pk)
+        submit_job_params = json_data['actions'][0]['parameters']
+        self.assertIn('token', submit_job_params)
+        self.assertEqual(job.submit_token.secret, submit_job_params['token'])
+
+    def test_getJobForBoard_adds_user_to_url(self):
+        panda_type = self.factory.ensure_device_type(name='panda')
+        self.factory.make_device(hostname='panda01', device_type=panda_type)
+        user = User.objects.create_user('test', 'e@mail.invalid', 'test')
+        user.save()
+        definition = {
+            'actions': [
+                {
+                    "command": "submit_results",
+                    "parameters":
+                    {
+                        "server": "http://test-server/RPC2/",
+                        "stream": "/private/personal/test/test/",
+                    }
+                }
+            ]
+        }
+        job = self.factory.make_testjob(
+            requested_device_type=panda_type, submitter=user,
+            definition=json.dumps(definition))
+        transaction.commit()
+        json_data = DatabaseJobSource().getJobForBoard_impl('panda01')
+        job = TestJob.objects.get(pk=job.pk)
+        submit_job_params = json_data['actions'][0]['parameters']
+        self.assertEqual("http://test@test-server/RPC2/", submit_job_params['server'])
+
     def get_device_and_running_job(self):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(requested_device=device)
         transaction.commit()
         DatabaseJobSource().getJobForBoard_impl('panda01')
-        return device, job
+        return device, TestJob.objects.get(pk=job.pk)
 
-    def test_jobCompleted_set_statuses(self):
+    def test_jobCompleted_set_statuses_success(self):
         device, job = self.get_device_and_running_job()
         transaction.commit()
-        DatabaseJobSource().jobCompleted_impl('panda01')
+        DatabaseJobSource().jobCompleted_impl('panda01', 0)
         job = TestJob.objects.get(pk=job.pk)
         device = Device.objects.get(pk=device.pk)
         self.assertEqual(
             (Device.IDLE, TestJob.COMPLETE),
             (device.status, job.status))
 
+    def test_jobCompleted_set_statuses_failure(self):
+        device, job = self.get_device_and_running_job()
+        transaction.commit()
+        DatabaseJobSource().jobCompleted_impl('panda01', 1)
+        job = TestJob.objects.get(pk=job.pk)
+        device = Device.objects.get(pk=device.pk)
+        self.assertEqual(
+            (Device.IDLE, TestJob.INCOMPLETE),
+            (device.status, job.status))
+
     def test_jobCompleted_works_on_device_type_targeted(self):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(
             requested_device_type=device.device_type)
         transaction.commit()
         DatabaseJobSource().getJobForBoard_impl('panda01')
-        DatabaseJobSource().jobCompleted_impl('panda01')
+        DatabaseJobSource().jobCompleted_impl('panda01', 0)
         job = TestJob.objects.get(pk=job.pk)
         device = Device.objects.get(pk=device.pk)
         self.assertEqual(
@@ -362,7 +533,7 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         device, job = self.get_device_and_running_job()
         before = datetime.datetime.now()
         transaction.commit()
-        DatabaseJobSource().jobCompleted_impl('panda01')
+        DatabaseJobSource().jobCompleted_impl('panda01', 0)
         after = datetime.datetime.now()
         job = TestJob.objects.get(pk=job.pk)
         self.assertTrue(before < job.end_time < after)
@@ -370,10 +541,19 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
     def test_jobCompleted_clears_current_job(self):
         device, job = self.get_device_and_running_job()
         transaction.commit()
-        DatabaseJobSource().jobCompleted_impl('panda01')
+        DatabaseJobSource().jobCompleted_impl('panda01', 0)
         device = Device.objects.get(pk=device.pk)
         self.assertEquals(None, device.current_job)
 
+    def test_jobCompleted_deletes_token(self):
+        device, job = self.get_device_and_running_job()
+        token = job.submit_token
+        transaction.commit()
+        DatabaseJobSource().jobCompleted_impl('panda01', 0)
+        self.assertRaises(
+            AuthToken.DoesNotExist,
+            AuthToken.objects.get, pk=token.pk)
+
     def test_getLogFileForJobOnBoard_returns_writable_file(self):
         device, job = self.get_device_and_running_job()
         definition = {'foo': 'bar'}
